
/**************************************************************************************************************
*
*  Project     : Snapsys
*
*  Versions		 : 2.0
*
*  Created By  : George Leonard, george@mighty.co.za (always reachable here)
*  Date        : 15/07/2006
*  Modified	   : 26/11/2007, 	Changed fault for snapsys_sysnames from 'N' to 'L', fits in better with the
* 														new version of sp_snapsys. New way getmode acts with active=L
*
*  Description : Creates tables and objects required by sp_snapsys
*
***************************************************************************************************************/



/**************************************************************************************************************
*
*  INSTALLATIONS:
*  	Dependency: Tablespace audit_small_data, audit_small_indexes
*
*********************************************************************************************/
--connect system

CREATE TABLESPACE audit_small_data
    DATAFILE '+data3' SIZE 100m AUTOEXTEND ON NEXT 100m MAXSIZE UNLIMITED,
             '+data3' SIZE 100m AUTOEXTEND ON NEXT 100m MAXSIZE UNLIMITED
    EXTENT MANAGEMENT LOCAL UNIFORM SIZE 160k
		SEGMENT SPACE MANAGEMENT AUTO ;

-- small defined indexes
CREATE TABLESPACE audit_small_indexes
    DATAFILE '+data3' SIZE 100m AUTOEXTEND ON NEXT 100m MAXSIZE UNLIMITED,
             '+data3' SIZE 100m AUTOEXTEND ON NEXT 100m MAXSIZE UNLIMITED
    EXTENT MANAGEMENT LOCAL UNIFORM SIZE 160k
		SEGMENT SPACE MANAGEMENT AUTO ;


create user snaps identified by snaps default tablespace audit_small_data temporary tablespace temp;
alter user snaps quota unlimited on audit_small_data;
alter user snaps quota unlimited on audit_small_indexes;
grant create session to snaps;
grant connect to snaps;
grant create procedure to snaps;
grant create sequence to snaps;
grant create trigger to snaps;
grant create public synonym to snaps;
grant resource to snaps;
grant select on v_$session to snaps;
grant select on v_$mystat to snaps;


connect snaps/snaps@raceres;

CREATE TABLE snapsys_sysnames (
   seq_num			 		number,
   name							varchar2(32),
   active_ind     	varchar2(1)	default 'L',
   diag_ind			 		varchar2(1) default 'N',
	 freedescription  varchar2(128)
) tablespace audit_small_data;

COMMENT ON TABLE snapsys_sysnames IS 'Different System names, grouping of systems running in the cor schema IE ACQ/SMAC/SPIF'
/
COMMENT ON COLUMN snapsys_sysnames.seq_num IS 'Sequential numbering'
/
COMMENT ON COLUMN snapsys_sysnames.name IS 'Short Name/Abbreviation'
/
COMMENT ON COLUMN snapsys_sysnames.active_ind IS 'On/Enabled=>R, Learn Mode=>L, Off=>N'
/
COMMENT ON COLUMN snapsys_sysnames.diag_ind IS 'system diagnostic tracing via dbms_support or dbms_profiler, On=>Y, Off/N'
/

create unique index sssn_pk on snapsys_sysnames(seq_num) tablespace audit_small_indexes;
create unique index sssn_uk1 on snapsys_sysnames(name) tablespace audit_small_indexes;
create index sssn_idx1 on snapsys_sysnames(name, active_ind, seq_num) tablespace audit_small_indexes;

PROMPT Creating Primary Key on 'snapsys_sysnames'
ALTER TABLE snapsys_sysnames
 ADD (CONSTRAINT sssn_PK PRIMARY KEY
  (seq_num))
/

PROMPT Creating Check Constraint on 'snapsys_sysnames'
ALTER TABLE snapsys_sysnames
 ADD (CONSTRAINT SSSN_CKY_ACTIVE_IND CHECK (active_ind IN ('R', 'L', 'N')))
/

PROMPT Creating Sequence 'SSSN_SEQ_NUM'
CREATE SEQUENCE SSSN_SEQ_NUM
 NOMAXVALUE
 NOMINVALUE
 NOCYCLE
 NOCACHE
/


PROMPT Creating Trigger 'SSSN_BIR_SEQ_NUM'
CREATE OR REPLACE TRIGGER SSSN_BIR_SEQ_NUM
 BEFORE INSERT
 ON snapsys_SYSNAMES
 FOR EACH ROW
 WHEN (NEW.seq_num IS NULL)
DECLARE
-- 	Author      : George Leonard
--   	Date        : 27 Janurary 2005
--   	Description : This trigger gets the next value from SSSN_SEQ_NUM sequence before INSERT

   v_retmesg         VARCHAR2(2000);

BEGIN
   SELECT SSSN_SEQ_NUM.NEXTVAL
     INTO :NEW.SEQ_NUM
     FROM DUAL;
EXCEPTION
   WHEN OTHERS THEN
		NULL;
END;
/
show err

CREATE TABLE snapsys_objnames (
   seq_num					number,
   sysseq_num				number,
   name			   			varchar2(128),
   active_ind       varchar2(1) default 'N',
   diag_ind					number default 0,
	 freedescription  varchar2(128)
) tablespace audit_small_data;

COMMENT ON TABLE snapsys_objnames IS 'Different Object Names to be monitored, IE Different Form names or different Stored Procedures, Packages or functions or Triggers'
/
COMMENT ON COLUMN snapsys_objnames.seq_num IS 'Sequential numbering'
/
COMMENT ON COLUMN snapsys_objnames.seq_num IS 'System ID (snapsys_sysnames.seq_num) to which this object belongs'
/
COMMENT ON COLUMN snapsys_objnames.name IS 'Object identifyer, IE Procedure name or package.procedure name or trigger name'
/
COMMENT ON COLUMN snapsys_objnames.active_ind IS 'On/Enabled=>Y, Off=>N for timing logging'
/
COMMENT ON COLUMN snapsys_objnames.diag_ind IS '1 for dbms_profiler start or 100466, 100468, 1004612, 100530 for tkprof tracing'
/

create unique index sson_pk on snapsys_objnames(seq_num) tablespace audit_small_indexes;
create unique index sson_uk1 on snapsys_objnames(sysseq_num, name) tablespace audit_small_indexes;

PROMPT Creating Primary Key on 'snapsys_objnames'
ALTER TABLE snapsys_objnames
 ADD (CONSTRAINT sson_PK PRIMARY KEY
  (seq_num))
/

ALTER TABLE snapsys_objnames ADD (
  CONSTRAINT SSON_SEQ_NUM_FK FOREIGN KEY (SYSSEQ_NUM)
    REFERENCES snapsys_SYSNAMES (SEQ_NUM));

PROMPT Creating Check Constraint on 'snapsys_objnames'
ALTER TABLE snapsys_objnames
 ADD (CONSTRAINT SSON_CKY_ACTIVE_IND CHECK (active_ind IN ('Y', 'N')))
/

PROMPT Creating Sequence 'SSON_SEQ_NUM'
CREATE SEQUENCE SSON_SEQ_NUM
 NOMAXVALUE
 NOMINVALUE
 NOCYCLE
 NOCACHE
/

PROMPT Creating Trigger 'SSON_BIR_SEQ_NUM'
CREATE OR REPLACE TRIGGER SSON_BIR_SEQ_NUM
 BEFORE INSERT
 ON snapsys_OBJNAMES
 FOR EACH ROW
 WHEN (NEW.seq_num IS NULL)
DECLARE
-- 	Author      : George Leonard
--   	Date        : 27 Janurary 2005
--   	Description : This trigger gets the next value from SSON_SEQ_NUM sequence before INSERT

   v_retmesg         VARCHAR2(2000);

BEGIN
   SELECT SSON_SEQ_NUM.NEXTVAL
     INTO :NEW.SEQ_NUM
     FROM DUAL;
EXCEPTION
   WHEN OTHERS THEN
		NULL;
END;
/

-- This table is going to fast and get BIG!!! maybe have a bigger extend (5m) based tablespace for it and associated indexes

CREATE TABLE snapsys_logs (
   objid						number,
   username					varchar2(32),
   call_time       		timestamp default systimestamp,
   fin_time       		timestamp default systimestamp,
 	notes						varchar2(1000)
) tablespace audit_small_data;

COMMENT ON TABLE snapsys_logs IS 'Records of for every time the object designated was called'
/
COMMENT ON COLUMN snapsys_logs.objid IS 'Seq_num from snapsys_objnames for object'
/
COMMENT ON COLUMN snapsys_logs.username IS 'User name executing the call, W Number'
/
COMMENT ON COLUMN snapsys_logs.call_time IS 'Call Start, Start of Form Paint or Procedure/Package/Trigger Start'
/
COMMENT ON COLUMN snapsys_logs.fin_time IS 'Call End, End of Form Paint or Procedure/Package/Trigger Start, The Delta to call_time is the job execution time'
/
COMMENT ON COLUMN snapsys_logs.notes IS 'Free Form Note'
/

create index slog_idx1 on snapsys_logs(objid, call_time, username) tablespace audit_small_indexes;


CREATE TABLE snapsys_errors (
    msg_date       		timestamp default systimestamp,
    msg_desc     			varchar2(1000)
) tablespace audit_small_data;

COMMENT ON TABLE snapsys_errors IS 'Output Catch for "Exception / When Others"'
/
COMMENT ON COLUMN snapsys_errors.msg_date IS 'Systimestamp when error occured'
/
COMMENT ON COLUMN snapsys_errors.msg_desc IS 'Ora Error number and description'
/

CREATE TABLE snapsys_objects (
    system_name      		varchar2(32),
    object_name				varchar2(256)
) tablespace audit_small_data;

COMMENT ON TABLE snapsys_objects IS 'This table gets populated when a system is marked as Learn mode in snapsys_sysnames, This information can then be used to populate snapsys_objnames'
/
COMMENT ON COLUMN snapsys_objects.system_name IS 'System Name'
/
COMMENT ON COLUMN snapsys_objects.object_name IS 'Calling Object identifier'
/

CREATE TABLE snapsys_avgs (
   objid						number,
   hourofday				varchar2(2),
   min_time					number(8,4),
   avg_time       		number(8,4),
   max_time					number(8,4),
   log_date					date default SYSDATE,
   notes						varchar2(1000)
) tablespace audit_small_data;

COMMENT ON TABLE snapsys_avgs IS 'This will be the base line values that is used to determine if object is running faster or slower than normal'
/
COMMENT ON COLUMN snapsys_avgs.objid IS 'seq_num from snapsys_objnames for object'
/
COMMENT ON COLUMN snapsys_avgs.hourofday IS 'If we care grouping baseline values per hour of day, the hour of the day the data is applicable to'
/
COMMENT ON COLUMN snapsys_avgs.min_time IS 'Min duration for object call'
/
COMMENT ON COLUMN snapsys_avgs.avg_time IS 'Avg duration for object call'
/
COMMENT ON COLUMN snapsys_avgs.max_time IS 'Max duration for object call'
/
COMMENT ON COLUMN snapsys_avgs.log_date IS 'Date baseline value was stored'
/
COMMENT ON COLUMN snapsys_avgs.notes IS 'User comment for baseline value'
/

create index ssav_idx1 on snapsys_avgs(objid) tablespace audit_small_indexes;
create index ssav_idx2 on snapsys_avgs(objid, hourofday) tablespace audit_small_indexes;

ALTER TABLE snapsys_avgs ADD (
   CONSTRAINT SSAV_SEQ_NUM_FK FOREIGN KEY (OBJID)
     REFERENCES snapsys_OBJNAMES (seq_num));

-- for testing and whatever else.
insert into snapsys_sysnames(name) values ('TST');
commit;

--!! Create Package

--create public synonym sp_snapsys for snaps.sp_snapsys;
--grant execute on snaps.sp_snapsys to public;

