/*

 1. simple once at star, once at end.
 2. 1. + multiple calls around code to be monitored.
 3. Forms method that does not support systimestamp variable type.

*/

prompt Creating Package testpack
	CREATE OR REPLACE PACKAGE TESTPACK
	AS
	   -- Called for Alter Drop of Tables
		PROCEDURE TEST1;
		PROCEDURE TEST2;
		PROCEDURE TEST3;

	END testpack;
	/

	CREATE OR REPLACE PACKAGE BODY TESTPACK
	AS
	-- Main example of what every package will have, a start end delta
	PROCEDURE test1
	IS
	   /* Variables */
	  l_ver   		VARCHAR2 (4);
		n_loopcnt   number;
	 	-- snapsys variables
		--d_start		timestamp default systimestamp;
		d_start		timestamp default systimestamp;
	BEGIN
		-- lets waist some time
		FOR n_loopcnt IN 1..1000000
		LOOP
			null;
		END LOOP;

	   -- lots of different code througout the package
	 	-- LAST CALL IN FORM, PACKAGE/PROCEDURE OR TRIGGER
		-- Call sp_snapsys to record the run time/delta from the time d_start was created
		sp_snapsys.snaplog('TST','TESTPACK.TEST1','george', d_start, '');

	END test1;

	-- Example 2, calling it not only at end for a start end delta but having additional step monitoring
	PROCEDURE test2
	IS
	   /* Variables */
	  l_ver   		VARCHAR2 (4);
		n_loopcnt   number;
	 	-- snapsys variables
		d_start	  	timestamp default systimestamp;
		d_start1		timestamp default systimestamp;
		d_start2		timestamp default systimestamp;
		d_start3		timestamp default systimestamp;
	BEGIN
	 	-- lets waist some time
		FOR n_loopcnt IN 1..1000000
		LOOP
			null;
		END LOOP;

		-- Call sp_snapsys to record the run time/delta from the time d_start was created
		sp_snapsys.snaplog('TST','TESTPACK.TEST2.1','', d_start1, 'point one in package');

	 	d_start2 := systimestamp;
		-- lets waist some time
		FOR n_loopcnt IN 1..100000
		LOOP
			null;
		END LOOP;
		-- Call sp_snapsys to record the run time/delta from the time d_start was created
		sp_snapsys.snaplog('TST','TESTPACK.TEST2.2','george', d_start2, 'point two in package');

	 	d_start3 := systimestamp;
		-- lets waist some time
		FOR n_loopcnt IN 1..100000
		LOOP
			null;
		END LOOP;
		-- Call sp_snapsys to record the run time/delta from the time d_start was created
		sp_snapsys.snaplog('TST','TESTPACK.TEST2.3','', d_start3, 'point three in package');
		sp_snapsys.snaplog('TST','TESTPACK.TEST2.POST','george', d_start, 'Package end');

	END test2;

	-- This is the method used when used with forms that dont support the systimestamp variable type
	-- test 3 calls the snaplog in the overloaded method/call using the getdate function and a varchar2
	PROCEDURE test3
	IS
	  /* Variables */
	  l_ver   		VARCHAR2 (4);
		n_loopcnt   number;
	 	-- snapsys variables
		--d_start		timestamp default systimestamp;
		v_start		varchar2(30);

	BEGIN

		v_start := sp_snapsys.getdate;
		-- lets waist some time
		FOR n_loopcnt IN 1..1000000
		LOOP
			null;
		END LOOP;

	   -- lots of different code througout the package
	 	-- LAST CALL IN FORM, PACKAGE/PROCEDURE OR TRIGGER
		-- Call sp_snapsys to record the run time/delta from the time d_start was created
		sp_snapsys.snaplog('TST','TESTPACK.TEST3','george', v_start, '');

	END test3;

END testpack;
/
